// =================================================================================================
// BareBasics - Minimal Horde3D Sample using GLFW
// =================================================================================================

#include "glfw.h"
#include "Horde3D.h"
#include "Horde3DUtils.h"

// Window Parameters
const int appWidth = 800;
const int appHeight = 600;
bool running = true;

// Horde3D Handles
H3DNode model;
H3DNode camera;

// =================================================================================================
// Horde3D Game Code
// =================================================================================================

bool initGame()
{
	// Initialize engine
	if( !h3dInit() ) return false;

	// Set size of the viewport
	h3dSetupViewport( 0, 0, appWidth, appHeight, true );

	// Enable smooth animations (frame interpolation)
	h3dSetOption( H3DOptions::FastAnimation, 0 );
	
	// Add pipeline resource
	H3DRes pipeRes = h3dAddResource( H3DResTypes::Pipeline, "pipelines/forward.pipeline.xml", 0 );
	// Add model resource
	H3DRes modelRes = h3dAddResource( H3DResTypes::SceneGraph, "models/man/man.scene.xml", 0 );
	// Add animation resource
	H3DRes animRes = h3dAddResource( H3DResTypes::Animation, "animations/man.anim", 0 );
	// Load added resources
	h3dutLoadResourcesFromDisk( "../Content" );
	
	// Add model to scene
	model = h3dAddNodes( H3DRootNode, modelRes );
	// Apply animation
	h3dSetupModelAnimStage( model, 0, animRes, 0, "", false );
	
	// Add light source
	H3DNode light = h3dAddLightNode( H3DRootNode, "Light1", 0, "LIGHTING", "SHADOWMAP" );
	// Set light position and radius
	h3dSetNodeTransform( light, 0, 0, 0, 0, 0, 0, 1, 1, 1 );
	h3dSetNodeParamF( light, H3DLight::RadiusF, 0, 50.0f );
	
	// Add camera
	camera = h3dAddCameraNode( H3DRootNode, "Camera", pipeRes );

	return true;
}

void gameLoop( float fps )
{
	static float t = 0;
	
	// Increase animation time
	t = t + 10.0f * (1 / fps);
	
	// Play animation
	h3dSetModelAnimParams( model, 0, t, 1.0f );
	
	// Rotate model
	h3dSetNodeTransform( model, 0, -1, -3,  // Translation
						 0, t, 0,           // Rotation
						 1, 1, 1 );         // Scale
										   
	// Render scene
	h3dRender( camera );
	
	// Finish rendering of frame
	h3dFinalizeFrame();
}

void releaseGame()
{
	// Release engine
	h3dRelease();
}


// =================================================================================================
// GLFW Window Management
// =================================================================================================

int windowCloseListener()
{
	running = false;
	return 0;
}


int main( int argc, char** argv )
{
	// Initialize GLFW
	glfwInit();
	if( !glfwOpenWindow( appWidth, appHeight, 8, 8, 8, 8, 24, 8, GLFW_WINDOW ) ) return -1;
	glfwSetWindowCloseCallback( windowCloseListener );
	
	// Initialize application and engine
	if( !initGame() ) return false;

	// Main game loop
	double prevTime = 0;
	while( running )
	{
		// Calculate framerate
		double newTime = glfwGetTime();
		float fps = 1.0f / (float)(newTime - prevTime);
		prevTime = newTime;
		
		gameLoop( fps );

		// Display rendered image on screen
		glfwSwapBuffers();
	}

	// Quit
	releaseGame();
	glfwTerminate();
	return 0;
}
